﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Data.SQLite;

namespace System.Data.SQLiteManager
{
    /// <summary>
    /// SELECT結果を表現します
    /// </summary>
    public class Result
    {
        /// <summary>
        /// 列の配列です
        /// </summary>
        public List<Row> Rows { get; private set; }

        /// <summary>
        /// 列名の配列です
        /// </summary>
        public string[] ColumnName { get; private set; }

        /// <summary>
        /// SELECT結果を列群から初期化します
        /// </summary>
        /// <param name="rows"></param>
        public Result(Row[] rows)
        {
            this.Rows = rows.ToList<Row>();
        }

        /// <summary>
        /// SELECT結果を列群から初期化します
        /// </summary>
        /// <param name="rows"></param>
        public Result(List<Row> rows)
        {
            this.Rows = rows;
        }

        /// <summary>
        /// SELECT結果を列名と値を表現する文字列群から初期化します
        /// </summary>
        /// <param name="columnName"></param>
        /// <param name="values"></param>
        public Result(string[] columnName, string[][] values)
        {
            this.Rows = new List<Row>();

            for (int i = 0; i < values.Length; i++)
            {
                this.Rows.Add(new Row(columnName, values[i]));
            }
        }
        /// <summary>
        /// SELECT結果をSQLiteのSELECT読み込みから初期化します
        /// </summary>
        /// <param name="reader"></param>
        public Result(SQLiteDataReader reader)
        {
            string[] columnName = new string[reader.FieldCount];

            for (int i = 0; i < reader.FieldCount; i++)
            {
                //列名を取得・格納します
                columnName[i] = reader.GetName(i);
            }
            this.ColumnName = columnName;

            this.Rows = new List<Row>();
            this.Rows.Clear();

            while (reader.Read())
            {
                //SELECT結果を１行ずつ読み込み

                Row row = new Row();
                for (int i = 0; i < reader.FieldCount; i++)
                {
                    row.Cells.Add(this.ColumnName[i], new Cell(reader[i].ToString()));
                }

                this.Rows.Add(row);
            }
        }

        /// <summary>
        /// 行を取得します
        /// </summary>
        /// <param name="index"></param>
        /// <returns></returns>
        public Row this[int index]
        {
            get { return this.Rows[index]; }
        }

        /// <summary>
        /// 行数を取得します。
        /// </summary>
        public int Length
        {
            get { return this.Rows.Count; }
        }
    }

    /// <summary>
    /// SELECTの行を表現します
    /// </summary>
    public class Row
    {
        /// <summary>
        /// セルの配列です
        /// </summary>
        public Dictionary<string, Cell> Cells { get; private set; }

        /// <summary>
        /// 行を初期化します
        /// </summary>
        public Row()
        {
            this.Cells = new Dictionary<string, Cell>();
        }

        /// <summary>
        /// 文字列群から行の初期化を行います
        /// </summary>
        /// <param name="columnName"></param>
        /// <param name="cells"></param>
        public Row(string[] columnName, string[] cells)
        {
            this.Cells = new Dictionary<string, Cell>();
            for (int i = 0; i < columnName.Length; i++)
            {
                //辞書に追加します
                this.Cells[columnName[i]] = new Cell(cells[i]);
            }
        }

        /// <summary>
        /// セル群から行の初期化を行います
        /// </summary>
        /// <param name="columnName"></param>
        /// <param name="cells"></param>
        public Row(string[] columnName, Cell[] cells)
        {
            this.Cells = new Dictionary<string, Cell>();
            for (int i = 0; i < columnName.Length; i++)
            {
                this.Cells[columnName[i]] = cells[i];
            }
        }

        /// <summary>
        /// 列順からセルを指定します（反復処理などで使用します）
        /// </summary>
        /// <param name="index">列順</param>
        /// <returns>セル</returns>
        public Cell this[int index]
        {
            get { return this.Cells.ToArray()[index].Value; }
        }

        /// <summary>
        /// 列名からセルを指定します
        /// </summary>
        /// <param name="key">列名</param>
        /// <returns>セル</returns>
        public Cell this[string key]
        {
            get { return this.Cells[key]; }
        }

        /// <summary>
        /// セル数を取得します
        /// </summary>
        public int Length
        {
            get { return this.Cells.Count; }
        }
    }

    /// <summary>
    /// SELECTのセルを表現します
    /// </summary>
    public class Cell
    {
        string value;

        /// <summary>
        /// セルの初期化を行います
        /// </summary>
        /// <param name="value"></param>
        public Cell(string value)
        {
            this.value = value;
        }

        public override string ToString()
        {
            return this.value;
        }

        #region 指定の型で値を取得する

        /// <summary>
        /// 指定の型で値を取得する
        /// </summary>
        public string String
        {
            get { return value; }
        }
        /// <summary>
        /// 指定の型で値を取得する
        /// </summary>
        public int Integer
        {
            get
            {
                int number;
                if (int.TryParse(value, out number))
                {
                    return number;
                }

                throw new FormatException("整数に変換できない値です。");
            }
        }
        /// <summary>
        /// 指定の型で値を取得する
        /// </summary>
        public float Float
        {
            get
            {
                float number;
                if (float.TryParse(value, out number))
                {
                    return number;
                }

                throw new FormatException("浮動小数点に変換できない値です。");
            }
        }
        /// <summary>
        /// 指定の型で値を取得する
        /// </summary>
        public double Double
        {
            get
            {
                double number;
                if (double.TryParse(value, out number))
                {
                    return number;
                }

                throw new FormatException("実数に変換できない値です。");
            }
        }

        #endregion
    }
}
