﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Data.SQLite;

namespace System.Data.SQLiteManager
{
    /// <summary>
    /// SQLiteデータベースを抽象化する（Singletonパターン）
    /// </summary>
    public class SQLiteConnector
    {   
        #region フィールドとプロパティ

        /// <summary>
        /// SQLの実体
        /// </summary>
        SQLiteConnection connection;

        /// <summary>
        /// クエリの実行
        /// </summary>
        SQLiteCommand command;

        /// <summary>
        /// トランザクション
        /// </summary>
        SQLiteTransaction transaction;

        /// <summary>
        /// 自分自身の実体
        /// </summary>
        static SQLiteConnector instance;

        /// <summary>
        /// Singletonのインタンス
        /// </summary>
        public static SQLiteConnector Instance
        {
            get
            {
                if (instance == null)
                {
                    throw new NullReferenceException("データベースが開かれていません。初期化してから利用してください。");
                }

                return instance;
            }
        }

        #endregion

        /// <summary>
        /// privateにすることで、外部からインスタンスを作成させない
        /// </summary>
        /// <param name="dbFilename">接続するSQLiteデータベースのファイル名</param>
        private SQLiteConnector(string dbFilename)
        {
            //データベースファイルに接続
            if ((this.connection = new SQLiteConnection("Data Source=" + dbFilename)) == null)
            {
                throw new SQLiteException("DBの接続に失敗しました。");
            }
            if ((this.command = this.connection.CreateCommand()) == null)
            {
                throw new SQLiteException("SQLの実行ができません。");
            }

            //データベースを開く
            this.connection.Open();
        }

        /// <summary>
        /// ObjectのDbTypeを求める
        /// </summary>
        /// <param name="unknown">DbTypeが不明なObject</param>
        /// <returns>Objectに対応するDbType</returns>
        DbType getDbType(object unknown)
        {
            //DB型を判定
            if (unknown is int)
            {
                //整数
                return DbType.Int32;
            }
            else if (unknown is string)
            {
                //文字列
                return DbType.String;
            }
            else if (unknown is float || unknown is double)
            {
                //実数
                return DbType.Double;
            }
            else
            {
                throw new ArgumentException("渡された引数の型が処理できない形式です");
            }
        }

        /// <summary>
        /// パラメータをセット（プリペアステートメント）
        /// </summary>
        /// <param name="sql">実行するSQL</param>
        /// <param name="arg">パラメータ</param>
        private void setParameter(string sql, object[] arg)
        {
            //SQLセット
            this.command.CommandText = sql;
            this.command.Parameters.Clear();

            for (int i = 0; i < arg.Length; i++)
            {
                //パラメータ作成
                SQLiteParameter parameter = this.command.CreateParameter();
                parameter.DbType = getDbType(arg[i]);
                parameter.Value = arg[i];

                //パラメータを追加
                this.command.Parameters.Add(parameter);
            }

            //パラメータをセット
            this.command.Prepare();
        }

        /// <summary>
        /// 戻りのないSQL（INSERT,DELETE,UPDATEなど）を実行する
        /// </summary>
        /// <param name="sql">実行するSQL</param>
        /// <param name="arg">パラメータ</param>
        public void ExecuteNonQuery(string sql, params object[] arg)
        {
            try
            {
                setParameter(sql,arg);
                this.command.ExecuteNonQuery();
            }
            catch (Exception ex)
            {
                throw new SQLiteException(ex.ToString());
            }
        }

        /// <summary>
        /// INSERTの実行でオートインクリメントされる場合、自動で割り振られた番号を取得
        /// </summary>
        /// <param name="sql">実行するSQL</param>
        /// <param name="arg">パラメータ</param>
        /// <returns>自動で割り振られた番号</returns>
        public int ExecuteInsert(string sql, params object[] arg)
        {
            //まずは、INSERTを実行
            ExecuteNonQuery(sql, arg);

            try
            {
                //AUTOINCREMENTされた値を取得
                int number;

                command.CommandText = "SELECT LAST_INSERT_ROWID()";
                using (SQLiteDataReader sdr = command.ExecuteReader())
                {
                    number = int.Parse(sdr[0].ToString());
                }

                return number;
            }
            catch (Exception ex)
            {
                throw new SQLiteException(ex.ToString());
            }
        }

        /// <summary>
        /// SELECTの実行
        /// </summary>
        /// <param name="sql">実行するSQL</param>
        /// <param name="arg">パラメータ</param>
        /// <returns>得られる結果</returns>
        public Result ExecuteReader(string sql, params object[] arg)
        {
            try
            {
                setParameter(sql, arg);

                using (SQLiteDataReader sdr = this.command.ExecuteReader())
                {
                    //DBからSELECT結果を読み込み
                    return new Result(sdr);
                }
            }
            catch (Exception ex)
            {
                throw new SQLiteException(ex.ToString());
            }
        }

        /// <summary>
        /// DBに接続する
        /// </summary>
        /// <param name="dbFilename">接続するSQLiteのDBファイル名</param>
        public static void Connect(string dbFilename)
        {
            instance = new SQLiteConnector(dbFilename);
        }

        /// <summary>
        /// DBから切断する
        /// </summary>
        public static void Disconnect()
        {
            if (instance == null)
            {
                throw new NullReferenceException("DBが接続されていないときに切断しようとしました。");
            }

            //閉じる
            instance.dispose();
            instance = null;
        }

        /// <summary>
        /// 資源を解放する
        /// </summary>
        private void dispose()
        {
            //コマンドを閉じる
            this.command.Dispose();

            //接続を切る
            this.connection.Close();
        }

        /// <summary>
        /// DBファイルを新規作成する
        /// </summary>
        /// <param name="dbFilename"></param>
        public static void CreateDB(string dbFilename)
        {
            System.IO.File.Create(dbFilename).Close();
        }

        #region トランザクション

        /// <summary>
        /// トランザクションのロールバック
        /// </summary>
        public void TransactionRollBack()
        {
            //トランザクションのロールバック
            this.transaction.Rollback();

            //トランザクションの開放
            this.transaction.Dispose();
            this.transaction = null;
        }

        /// <summary>
        /// トランザクションの開始
        /// </summary>
        public void TransactionStart()
        {
            //トランザクションの開始
            this.transaction = this.connection.BeginTransaction();
        }

        /// <summary>
        /// トランザクションのコミット
        /// </summary>
        public void TransactionCommit()
        {
            //トランザクションのコミット
            this.transaction.Commit();

            //トランザクションを開放
            this.transaction.Dispose();
            this.transaction = null;
        }

        #endregion
    }
}
